; (function () {
	"use strict";

	/*
	* FALLBACK POLYFILLS
	* (only within the scope of current tasks without universality)
	*/
	if (!document.querySelectorAll) {
		document.querySelectorAll = function (selectors) {
			var cnames = selectors.split(',');
			var elements = [];

			for (var i = 0; i < cnames.length; i++) {
				var refs = document.getElementsByClassName(
					cnames[i].replace(/^\s+|\s+$/g, '').replace(/^\./, '') // trim extra spaces and dot
				);

				for (var j = 0; j < refs.length; j++)
					elements.push(refs[j]);
			}

			return elements;
		};
	}

	var __setClass = function (el, cname, act) {
		act = act || "add";
	
		if (el.classList) { // for modern user-agents compatibility
			if (act === "add") el.classList.add(cname);
			else if (act === "remove") el.classList.remove(cname);
			else if (act === "toggle") el.classList.toggle(cname);
		} else { // to legacy support
			var regExp = new RegExp("(^|\\s)" + cname + "(\\s|$)", "g");
			var hasClass = regExp.test(el.className);
			
			if (act === "add" && !hasClass) {
				el.className += " " + cname;
			} else if (act === "remove" && hasClass) {
				el.className = el.className.replace(regExp, " ").replace(/\s+/g, " "); // and trim extra spaces
			} else if (act === "toggle") {
				__setClass(el, cname, hasClass ? "remove" : "add");
			}
		}
	}
	
	/****************
	* DAILY WALLPAPER
	*****************/
	// Namespace for widget with grouping in global scope
	var DailyWallpaper = window.DailyWallpaper || {};

	DailyWallpaper.Widget = function (options) {	// constructor
		options = options || {}; // for user-agents that do not support default values in the parameters
		var self = this; // contextualization on the widget

		/*
		* PRIVATE MEMBERS
		*/
		self._MESSAGES = {
			NAME: "DailyWallpaper: ",
			ERR: {
				NO_ELEMENTS: { code: 1001, msg: " No elements found for selector " },
				NO_API: { code: 1002, msg: " No API found " },
				DATA_OBTAIN: { code: 1003, msg: " Failed to obtain data from the data provider. Response status code: " },
				DATA_TIMEOUT: { code: 1004, msg: " Request timed out. Remaining retries: " },
				DATA_INVALID: { code: 1005, msg: " Incorrect data received: " }
			}
		};
		self._attachTo = options.attachTo || ".dwp";
		self._pathToAPI = options.pathToAPI || null;
		self._obtainAttempts = isNaN(Number(options.obtainAttempts)) ? 3 : Number(options.obtainAttempts);
		self._obtainAwait = isNaN(Number(options.obtainAwait)) ? 5000 : Number(options.obtainAwait);
		self._fallbackImg = options.fallbackImg || null;
		self._instances = null;

		if (options.fxNames) {
			self._fxNames = options.fxNames.split(',');
			for (var i = 0; i < self._fxNames.length; i++) {
				self._fxNames[i] = self._fxNames[i].replace(/^\s+|\s+$/g, '');
			}
		} else {
			self._fxNames = [];
		}

		/* 
		* PRIVATE FUNCTIONS
		*/
		// Helpers
		var showErr = function (msg) {
			if (self._instances.length) {
				for (var i = 0; i < self._instances.length; i++) {
					self._instances[i].textContent = msg;
					__setClass(self._instances[i], "dwp--error", "add");
				}
			} else {
				var errElem = document.createElement("div");
				errElem.textContent = self._MESSAGES.NAME + msg;
				__setClass(errElem, "dwp--error", "add");
				document.body.appendChild(errElem);
			}

			console.error(self._MESSAGES.NAME + msg);
		}

		// Widget functions
		var init = function () {
			self._instances = document.querySelectorAll(self._attachTo);

			if (self._instances.length) {
				for (var i = 0; i < self._instances.length; i++) {
					__setClass(self._instances[i], "dwp", "add");
				}
			} else {
				showErr(self._MESSAGES.ERR.NO_ELEMENTS.code + self._MESSAGES.ERR.NO_ELEMENTS.msg + self._attachTo);
				return false;
			}

			if (!self._pathToAPI) {
				showErr(self._MESSAGES.ERR.NO_API.code + self._MESSAGES.ERR.NO_API.msg + self._pathToAPI);
			}

			return true;
		}

		var getDailyURL = function () {
			var req = new XMLHttpRequest();
			req.open("GET", self._pathToAPI, true);
			req.timeout = self._obtainAwait;

			req.ontimeout = function () {
				showErr(self._MESSAGES.ERR.DATA_TIMEOUT.code + self._MESSAGES.ERR.DATA_TIMEOUT.msg + self._obtainAttempts);
				
				if (self._obtainAttempts > 0) {
					self._obtainAttempts--;
					setTimeout(getDailyURL, 1000);
				} else {
					if (self._fallbackImg) {
						setWallpaper(self._fallbackImg);
					} else {
						showErr(self._MESSAGES.ERR.DATA_OBTAIN.code + self._MESSAGES.ERR.DATA_OBTAIN.msg + "-1 (timeout)");
					}
				}
			};

			req.onreadystatechange = function () {
				if (req.readyState === 4) {
					if (req.status >= 200 && req.status < 300) {
						setWallpaper(req.responseText);
					} else if (req.status >= 500 && req.status <= 504 && self._obtainAttempts > 0) {
						showErr(self._MESSAGES.ERR.DATA_TIMEOUT.code + self._MESSAGES.ERR.DATA_TIMEOUT.msg + self._obtainAttempts);
						self._obtainAttempts--;
						setTimeout(getDailyURL, 1000);
					} else if (req.status === 404) {
						if (self._fallbackImg) {
							setWallpaper(self._fallbackImg);
						} else {
							showErr(self._MESSAGES.ERR.DATA_OBTAIN.code + self._MESSAGES.ERR.DATA_OBTAIN.msg + req.status);
						}
					} else {
						showErr(self._MESSAGES.ERR.DATA_OBTAIN.code + self._MESSAGES.ERR.DATA_OBTAIN.msg + req.status);
					}
				}
			}

			req.send();
		}

		var setWallpaper = function (url) {
			if (!url) {
				if (self._fallbackImg) {
					url = self._fallbackImg;
				} else {
					showErr(self._MESSAGES.ERR.DATA_INVALID.code + self._MESSAGES.ERR.DATA_INVALID.msg + url);
					return;
				}
			}
			
			if (self._instances.length) {
				for (var i = 0; i < self._instances.length; i++) {
					self._instances[i].textContent = ""; // clearing all errors displayed in UI elements
					__setClass(self._instances[i], "dwp--error", "remove");
					__setClass(self._instances[i], "dwp--success", "add"); // remove loader

					self._instances[i].style.backgroundImage = "url(" + url + ")";

					if (self._fxNames.length) {
						for (var j = 0; j < self._fxNames.length; j++) {
							__setClass(self._instances[i], "dwp--fx-" + self._fxNames[j], "add");
						}
					}
				}
			}
		}

		if (init()) {
			getDailyURL();
		}
	};

	/* 
	* PUBLIC FUNCTIONS
	*/
	DailyWallpaper.Widget.prototype.toggleFX = function () {
		var self = this;
		if (self._instances.length && self._fxNames.length) {
			for (var i = 0; i < self._instances.length; i++) {
				for (var j = 0; j < self._fxNames.length; j++) {
					__setClass(self._instances[i], "dwp--fx-" + self._fxNames[j], "toggle");
				}
			}

		}
	};

	// expose reference to the global scope
	window.DailyWallpaper = DailyWallpaper;
})();